module Main where

import Prelude qualified as P
import Data.Foldable (toList)
import Text.Pandoc.Shared (isHeaderBlock)
import Text.Pandoc.Definition (Pandoc(Pandoc), Block(Header))

import Achille as A
import Achille.Prelude
import Achille.Yaml
import Achille.Stache

import Config
import Publications


data NavItem = NavItem
  { title :: Text
  , url   :: Text
  } deriving (Eq, Generic, Binary, ToJSON)

data Meta = Meta
  { title       :: Text
  , description :: Text
  } deriving (Eq, Generic, Binary, FromJSON, ToJSON)

data SiteContent = Content
  { meta    :: Meta
  , nav     :: [NavItem]
  , body    :: Text
  , pubs    :: [PubGroup]
  } deriving (Generic, ToJSON)


main = achille A.do
  index <- loadTemplate "index.mustache"

  match "assets/*" copy
  match "papers/*" copy
  match "static/*" copy

  pubs <- readYaml "publications.yaml"

  meta :*: nav :*: content <- A.do
    meta :*: doc <- readPandoc "index.md"
    meta :*: (getNav <$> doc) :*: renderPandoc doc

  Content <$> meta <*> nav <*> content <*> pubs
    & applyTemplate index
    & write "index.html"

  where getNav :: Pandoc -> [NavItem]
        getNav (Pandoc _ blocks) =
          let hdToItem (Header _ (id, _, _) _) = NavItem id ("/#" <> id)
          in hdToItem <$> toList (filter isHeaderBlock blocks)
